package com.nationrel.modules.alta.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.nationrel.modules.alta.entity.AltaAnalysisInfo;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.alta.service.AltaAnalysisInfoService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import jakarta.servlet.http.HttpServletRequest;
import java.util.Date;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import com.nationrel.modules.alta.handler.GlobalExceptionHandler;

/**
 * @author hcs on 2020/9/14.
 */
@Tag(name = "寿命数据分析-分析部分")
@RestController
@RequestMapping("/alta/analysis/info")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaAnalysisInfoController {

    private final AltaAnalysisInfoService analysisService;

    @Operation(summary = "试验数据")
    @GetMapping("list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "altaId", description = "altaId", in = ParameterIn.QUERY),
            @Parameter(name = "altaName", description = "分析名称", in = ParameterIn.QUERY),
            @Parameter(name = "startDate", description = "开始时间", in = ParameterIn.QUERY),
            @Parameter(name = "endDate", description = "结束时间", in = ParameterIn.QUERY),
            @Parameter(name = "pageIndex", description = "当前页码", in = ParameterIn.QUERY),
            @Parameter(name = "pageSize", description = "每页条数", in = ParameterIn.QUERY)
    })
    public Result<IPage<AltaAnalysisInfo>> analysisList(@RequestParam(required = false) String altaId,
                                                        @RequestParam(required = false) String altaName,
                                                        @RequestParam(required = false) Date startDate,
                                                        @RequestParam(required = false) Date endDate,
                                                        @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                                                        @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.listByPage(userInfo, altaId, altaName, startDate, endDate, pageIndex, pageSize);
    }

    @Operation(summary = "分析创建")
    @PostMapping("add")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> createAnalysis(@RequestBody AltaAnalysisInfo entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getAltaName())) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.addAnalysis(userInfo, entity);
    }

    @Operation(summary = "分析修改")
    @PostMapping("edit")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> updateAnalysis(@RequestBody AltaAnalysisInfo entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getAltaName())) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.updateAnalysisInfo(userInfo, entity);
    }

    @Operation(summary = "分析删除")
    @DeleteMapping("/delete")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> deleteAnalysis(@RequestParam(name = "altaId", required = true) String altaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(altaId)) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.数据ID不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.deleteAnalysisInfo(userInfo, altaId);
    }

}
