package com.nationrel.modules.alta.controller;

import com.nationrel.modules.alta.dto.AltaColumnsEditDto;
import com.nationrel.modules.alta.entity.AltaColumn;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.service.AltaColumnAndDataService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import com.nationrel.modules.alta.handler.GlobalExceptionHandler;

/**
 * @author hcs on 2021/9/14.
 */
@Tag(name = "加速寿命实验-columns部分")
@RestController
@RequestMapping("/alta/columns")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaColumnController {

    private final AltaColumnAndDataService columnAndDataService;

    @Operation(summary = "columns数据")
    @GetMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析Id", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<AltaColumn>> analysisList(HttpServletRequest request, @RequestParam(required = false) String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return columnAndDataService.listColumn(analysisId, userInfo.getLanguageKey());
    }

    @Operation(summary = "columns数据")
    @GetMapping("/select")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析Id", in = ParameterIn.QUERY)
    })
    public ServerResponse<Map> selectColumns(HttpServletRequest request, @RequestParam(required = false) String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return columnAndDataService.listSelectColumn(analysisId, userInfo.getLanguageKey());
    }

    @Operation(summary = "columns修改")
    @PutMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateAnalysis(HttpServletRequest request, @RequestBody AltaColumnsEditDto entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return columnAndDataService.updateColumns(userInfo, entity);
    }
}
