// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.alta.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.alta.mapper.*;
import com.nationrel.modules.alta.constant.AltaConstants;
import com.nationrel.modules.alta.dto.AltaCalDataDto;
import com.nationrel.modules.alta.dto.AltaFastCalculateDto;
import com.nationrel.modules.alta.entity.*;
import com.nationrel.modules.alta.enums.AltaDistributingCodeEnum;
import com.nationrel.modules.alta.enums.AltaLifeStressModelEnum;
import com.nationrel.modules.alta.vo.AltaAnalysisCalcResultVo;
import com.nationrel.modules.alta.vo.AltaAnalysisCalcVo;
import com.nationrel.modules.alta.vo.AltaCalcResultVo;
import com.nationrel.modules.alta.vo.AltaCalcResultsVo;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.alta.AltaCommonUsedEnum;
import com.nationrel.modules.alta.general.domain.enums.alta.AltaCustomPromptEnum;
import com.nationrel.modules.alta.general.domain.enums.common.WhetherFlagEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.alta.general.exception.DescribeException;
import com.nationrel.modules.alta.general.util.common.BasicDataUtil;
import com.nationrel.modules.alta.general.util.common.DateTimeUtil;
import com.nationrel.modules.alta.general.util.common.ValueWidgetUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName AltaCalcResultService
 * @Description 计算数据
 * @Author hecs
 * @CreateDate 2021-04-11
 * @Version V1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaCalcResultService {

    private final AltaAnalysisCalcResultMapper analysisCalcResultMapper;
    private final AltaAnalysisInfoMapper analysisInfoMapper;
    private final AltaCalcResultMapper calcResultMapper;
    private final AltaAnalysisMapper analysisMapper;
    private final AltaRgaApiService rgaApiService;
    private final AltaSettingMapper settingMapper;
    private final AltaColumnMapper columnMapper;
    private final AltaDataMapper dataMapper;
    private final AltaPlotMapper plotMapper;

    /**
     * 计算结果
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<AltaCalcResultsVo> calcResult(HeaderUserInfo userInfo, AltaCalDataDto entity) {
        AltaCalcResultsVo result = new AltaCalcResultsVo();
        String analysisId = entity.getAnalysisId();
        StringBuilder columnSetting = new StringBuilder();
        for (int i = 0; i < entity.getColumnIds().size(); i++) {
            String altaColumn = entity.getColumnIds().get(i).getColumnId();
            if (!"0".equals(altaColumn)) {
                columnSetting.append(",").append(entity.getColumnIds().get(i).getColumnId()).append(",").append(entity.getColumnIds().get(i).getStressLevel());
            }
            // 若是计算的数据是GeneralLogLinear，则需要添加generalCode配置
            if (entity.getLifeStressModel().toString().equals(AltaLifeStressModelEnum.GeneralLogLinear.getIndex())) {
                columnSetting.append(",").append(entity.getColumnIds().get(i).getGeneralCode());
            }
        }
        // 更新选择模型
        AltaSetting setting = settingMapper.selectById(analysisId);
        boolean deleteAnalysisResultData = false;
        // 当设置发生改变时更新设置
        if (!setting.getDistributionType().equals(entity.getDistributionType()) || !setting.getLifeStressModel().equals(entity.getLifeStressModel())) {
            setting.setUpdateBy(userInfo.getUserName());
            setting.setDistributionType(entity.getDistributionType());
            setting.setLifeStressModel(entity.getLifeStressModel());
            settingMapper.updateById(setting);
            AltaColumnAndDataService.deleteAnalysisResultData(analysisId, calcResultMapper, settingMapper);
        }
        List<AltaCalcResultVo> columnResults = new ArrayList<>();
        List<AltaCalcResultVo> settingResults = new ArrayList<>();
        settingResults.add(new AltaCalcResultVo(userInfo.getLanguageKey().equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? "分布" : "Distribution", AltaDistributingCodeEnum.getNameByLanguageKey(entity.getDistributionType().toString(), userInfo.getLanguageKey())));
        settingResults.add(new AltaCalcResultVo(userInfo.getLanguageKey().equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? "模型" : "Module", AltaLifeStressModelEnum.getNameByLanguageKey(entity.getLifeStressModel().toString(), userInfo.getLanguageKey())));
        columnResults.add(new AltaCalcResultVo(userInfo.getLanguageKey().equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? "应力项" : "Stress Item", userInfo.getLanguageKey().equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? "使用值" : "Use Value"));
        result.setSettingResults(settingResults);
        AltaCalcResult calcResult = calcResultMapper.selectOne(new QueryWrapper<AltaCalcResult>().lambda()
                .eq(AltaCalcResult::getAnalysisId, entity.getAnalysisId())
                .eq(AltaCalcResult::getDistributionType, entity.getDistributionType())
                .eq(AltaCalcResult::getColumnSetting, columnSetting.toString())
                .eq(AltaCalcResult::getLifeStressModel, entity.getLifeStressModel()));
        // 查询原有的设置信息
        List<AltaColumn> preColumns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, entity.getAnalysisId()).eq(AltaColumn::getSelectStatus, 1));
        if (!ValueWidgetUtil.isNullOrEmpty(calcResult)) {
            result.setBeta(calcResult.getBeta());
            result.setLogSigma(calcResult.getLogSigma());
            result.setGamma(calcResult.getGamma());
            result.setEta(calcResult.getEta());
            result.setDistributionType(calcResult.getDistributionType());
            result.setBoundsFlag(entity.getLifeStressModel().toString().equals(AltaLifeStressModelEnum.GeneralLogLinear.getIndex()) ? judgeBoundsFlag(calcResult.getCovMatrix()) : WhetherFlagEnum.YES.getIndex());
            result.setCalcResults(getcalcResultVos(entity.getLifeStressModel(), entity.getDistributionType(), calcResult));
            preColumns.stream().filter(column -> column.getShowStatus() == 1).map(column -> new AltaCalcResultVo(column.getName() + "(" + column.getUnit() + ")", column.getStressLevel().toString())).forEach(columnResults::add);
            result.setColumnResults(columnResults);
            return ServerResponse.createBySuccess(result);
        }
        // 先更新所有列为未选择
        columnMapper.update(AltaColumn.builder().showStatus(0).build(), new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId));
        // 记录前端选择需要计算的应力
        List<String> stressList = new ArrayList<>();
        List<String> showStressList = new ArrayList<>();
        // 处理已经选中的信息
        HashMap<String, String> selectColumns = new HashMap<>();
        for (int i = 0; i < entity.getColumnIds().size(); i++) {
            String altaColumn = entity.getColumnIds().get(i).getColumnId();
            // columnId为"0"表示未选择此应力计算
            if (!"0".equals(altaColumn)) {
                stressList.add(entity.getStressTf().get(i));
                showStressList.add(entity.getColumnIds().get(i).getColumnId());
                selectColumns.put(altaColumn, StringUtils.isBlank(entity.getColumnIds().get(i).getGeneralCode()) ? BasicDataUtil.EMPTY_STR : entity.getColumnIds().get(i).getGeneralCode());
            }
        }
        // 判断变化以及更新数据
        for (AltaColumn column : preColumns) {
            // 判断当前已选择的数据是否发生变化，并更新最新的变化
            if (selectColumns.get(column.getColumnId()) != null) {
                columnResults.add(new AltaCalcResultVo(column.getName() + "(" + column.getUnit() + ")", column.getStressLevel().toString()));
                AltaColumn columnUpdate = AltaColumn.builder().showStatus(1).build();
                // 进入此处判断则说明原来未选择该应力进行分析但是现在选择了，或者是GGL选择的generalCode不同，那么就需要删除原有的中间数据
                if (column.getShowStatus() != 1 || (entity.getLifeStressModel().toString().equals(AltaLifeStressModelEnum.GeneralLogLinear.getIndex()) && !selectColumns.get(column.getColumnId()).equals(column.getGeneralCode()))) {
                    deleteAnalysisResultData = true;
                    columnUpdate.setGeneralCode(selectColumns.get(column.getColumnId()));
                }
                // 更新column的设置数据
                columnMapper.update(columnUpdate, new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getColumnId, column.getColumnId()));
            } else {
                // 修改当前未选择的数据，如发生变化，进行修改
                deleteAnalysisResultData = column.getShowStatus() == 1 ? true : deleteAnalysisResultData;
            }
        }
        if (deleteAnalysisResultData) {
            AltaColumnAndDataService.deleteAnalysisResultData(analysisId, calcResultMapper, settingMapper);
        }
        result.setColumnResults(columnResults);
        entity.getStressTf().clear();
        entity.getStressTf().addAll(stressList);
        this.validTableData(analysisId, showStressList, setting.getLifeStressModel(), userInfo.getLanguageKey());

        // 准备数据
        List<AltaData> dataList = dataMapper.selectList(new QueryWrapper<AltaData>().lambda().eq(AltaData::getAnalysisId, analysisId));
        // 获取当前选择的列
        List<AltaColumn> altaColumns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getShowStatus, "1"));

        String param;
        Map<String, Object> requestMap = rgaApiService.assemblyData(altaColumns, dataList, setting);
        param = JSONObject.toJSONString(requestMap);
        JSONObject jsonObject = new JSONObject();
        // jsonObject.put("LSR",
        // AltaLifeStressModelEnum.getName(param.getLifeStressModel()));
        String uri = rgaApiService.getAltaUri(entity.getDistributionType(), entity.getLifeStressModel());

        // GLL请求格式不一样
        if (setting.getLifeStressModel().toString().equals(AltaLifeStressModelEnum.GeneralLogLinear.getIndex())) {
            Map<String, Object> map = new HashMap<>();
            map.put("Data", requestMap);
            map.put("StressTF", entity.getStressTf());
            param = JSONObject.toJSONString(map);
        }
        String resultsCallApi = rgaApiService.getResultsCallApi(jsonObject, uri, AltaConstants.RGA_API_CONTENTTYPE_1, AltaConstants.RGA_API_ACCEPTTYPE_1, param);
        //System.err.println(resultsCallApi);计算失败请检查数据或设置
        if (StringUtils.isEmpty(resultsCallApi) || resultsCallApi.indexOf("error") > 0) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.计算失败请检查数据或设置, userInfo.getLanguageKey());
        } else {
            calcResultMapper.delete(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId).eq(AltaCalcResult::getDistributionType, entity.getDistributionType()).eq(AltaCalcResult::getLifeStressModel, entity.getLifeStressModel()));
            JSONObject obj = JSON.parseObject(resultsCallApi);
            if (StringUtils.isNotBlank(AltaDistributingCodeEnum.getIndexByEnglishName(obj.getString("distributionType")))) {
                obj.put("distributionType", AltaDistributingCodeEnum.getIndexByEnglishName(obj.getString("distributionType")));
            }
            if (StringUtils.isNotBlank(AltaLifeStressModelEnum.getIndexByApiName(obj.getString("lifeStressModel")))) {
                obj.put("lifeStressModel", AltaLifeStressModelEnum.getIndexByApiName(obj.getString("lifeStressModel")));
            }
            calcResult = obj.toJavaObject(AltaCalcResult.class);
//            calcResult = JSONObject.parseObject(resultsCallApi, AltaCalcResultApiResp.class).getAltaCalcResult();
            calcResult.setAnalysisId(analysisId);
            calcResult.setLifeStressModel(entity.getLifeStressModel());
            calcResult.setDistributionType(entity.getDistributionType());
            calcResult.setColumnSetting(columnSetting.toString());
            calcResultMapper.insert(calcResult);
            // 计算后清除画图
            plotMapper.delete(new QueryWrapper<AltaPlot>().lambda().eq(AltaPlot::getAnalysisId, analysisId)
                    .eq(AltaPlot::getDistributionType, entity.getDistributionType())
                    .eq(AltaPlot::getLifeStressModel, entity.getLifeStressModel()));
            result.setBeta(calcResult.getBeta());
            result.setLogSigma(calcResult.getLogSigma());
            result.setDistributionType(calcResult.getDistributionType());
            result.setCalcResults(getcalcResultVos(entity.getLifeStressModel(), entity.getDistributionType(),
                    calcResult));
            result.setBoundsFlag(entity.getLifeStressModel().toString().equals(AltaLifeStressModelEnum.GeneralLogLinear.getIndex()) ? judgeBoundsFlag(calcResult.getCovMatrix()) : WhetherFlagEnum.YES.getIndex());
            AltaAnalysisInfoService.updateUpdateInfo(userInfo, analysisMapper.selectById(analysisId).getAltaId(),
                    analysisInfoMapper);
            return ServerResponse.createBySuccess(result);
        }
    }


    /**
     * 验证当前的数据在所选模型下是否能够计算
     *
     * @param analysisId
     * @param columnIds
     * @param stressModel
     * @param languageKey
     */
    private void validTableData(String analysisId, List<String> columnIds, Integer stressModel, Integer languageKey) {
        int countF = dataMapper.countfCompleteByFtEqu(analysisId);
        // 只能选择单应力的模型
        if (stressModel.toString().equals(AltaLifeStressModelEnum.Arrhenius.getIndex()) || stressModel.toString().equals(AltaLifeStressModelEnum.Eyring.getIndex()) || stressModel.toString().equals(AltaLifeStressModelEnum.InversePowerLaw.getIndex())) {
            if (columnIds.size() != 1)
                // 只能选择一个应力才能分析
            {
                throw new DescribeException(AltaCustomPromptEnum.只能选择一个应力, languageKey, ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
            if (countF < 3 || dataMapper.countStressCompleteById(analysisId, columnIds.get(0)) < 2)
                // 至少需要3条失效数据才能分析
            {
                throw new DescribeException(AltaCustomPromptEnum.至少需要3条失效数据, languageKey, ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
            // 只能选择双应力的模型
        } else if (stressModel.toString().equals(AltaLifeStressModelEnum.TemAndSid.getIndex()) || stressModel.toString().equals(AltaLifeStressModelEnum.TemAndNotHot.getIndex())) {
            if (columnIds.size() != 2)
                // 只能选择2个应力才能分析
            {
                throw new DescribeException(AltaCustomPromptEnum.只能选择二个应力, languageKey, ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
            if (countF < 4 || dataMapper.countGroupStressById(analysisId, BasicDataUtil.listToString(columnIds)) < 3)
                // 至少需要3条失效数据才能分析
            {
                throw new DescribeException(AltaCustomPromptEnum.至少需要4条失效数据, languageKey, ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        } else {
            if (countF < columnIds.size() + 2 || dataMapper.countGroupStressById(analysisId, BasicDataUtil.listToString(columnIds)) <= columnIds.size()) {
                // 应力值组合数＞应力类型数；失效数F＞=应力类型数＋2
                throw new DescribeException(AltaCustomPromptEnum.GGL需要的计算规则, languageKey, ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        }
    }

    /**
     * 判断当前数据当前设置下是否可以画上下曲线
     *
     * @param covMatrix
     * @return
     */
    public static Integer judgeBoundsFlag(String covMatrix) {
        if (StringUtils.isBlank(covMatrix)) {
            return WhetherFlagEnum.YES.getIndex();
        }
        List<List> covMatrixValues = JSONObject.parseArray(covMatrix, List.class);
        for (int i = 0; i < covMatrixValues.size(); i++) {
            if (covMatrixValues.get(i).get(i).toString().contains("-")) {
                return WhetherFlagEnum.NO.getIndex();
            }
        }
        return WhetherFlagEnum.YES.getIndex();
    }

    /**
     * 生成前端需要的vo数据
     *
     * @param lifeStressModel
     * @param distributionType
     * @param calcResult
     * @return
     */
    public static List<AltaCalcResultVo> getcalcResultVos(Integer lifeStressModel, Integer distributionType,
                                                          AltaCalcResult calcResult) {
        List<AltaCalcResultVo> result = new ArrayList<>();
        if (ValueWidgetUtil.isNullOrEmpty(calcResult)) {
            return result;
        }
        // 若应力模型为 0:Arrhenius  1:Eyring  2:IPL  3:Temp_Humidity  4:Temp_NonThermal
        if (lifeStressModel.equals(0) || lifeStressModel.equals(1) || lifeStressModel.equals(2) || lifeStressModel.equals(3) || lifeStressModel.equals(4)) {
            // 若分布类型为 0:Weibull返回数据则包含β，否则包含Std
            result.add(distributionType.equals(0) ? new AltaCalcResultVo("β", getScientificResult(calcResult.getBeta())) : new AltaCalcResultVo("Std", getScientificResult(calcResult.getLogSigma())));
            // 若应力模型不为 2:IPL，则返回数据包含Ea
            if (!lifeStressModel.equals(2)) {
                result.add(new AltaCalcResultVo("Ea", getScientificResult(calcResult.getEa())));
            }
            // 其余只要满足头部条件则添加LK值、θ(0)、θ(1)
            result.add(new AltaCalcResultVo("LK", getScientificResult(calcResult.getLkv())));
            result.add(new AltaCalcResultVo("θ(0)", getScientificResult(calcResult.getTheta0())));
            result.add(new AltaCalcResultVo("θ(1)", getScientificResult(calcResult.getTheta1())));
            if (lifeStressModel.equals(3) || lifeStressModel.equals(4))
                // 若应力模型不为 2:IPL 4:Temp_NonThermal，则返回数据包含θ(2)
            {
                result.add(new AltaCalcResultVo("θ(2)", getScientificResult(calcResult.getTheta2())));
            }
        }
        // 若应力模型为 5:General_Log_Linear
        if (lifeStressModel.equals(5)) {
            // 若分布类型为 0:Weibull返回数据则包含β，否则包含Std
            result.add(distributionType.equals(0) ? new AltaCalcResultVo("β", getScientificResult(calcResult.getBeta())) : new AltaCalcResultVo("Std", getScientificResult(calcResult.getLogSigma())));
            String[] alphas = calcResult.getAlpha().replace("[", "").replace("]", "").split(",");
            // 其余只要满足头部条件则添加LK值、θ(0)、θ(1)、θ(2)、α
            for (int i = 0; i < alphas.length; i++) {
                result.add(new AltaCalcResultVo("α(" + i + ")", getScientificResult(alphas[i])));
            }
            result.add(new AltaCalcResultVo("LK", getScientificResult(calcResult.getLkv())));
//            result.add(new AltaCalcResultVo("θ(0)", getEResult(calcResult.getTheta0())));
//            result.add(new AltaCalcResultVo("θ(1)", getEResult(calcResult.getTheta1())));
//            result.add(new AltaCalcResultVo("θ(2)", getEResult(calcResult.getTheta2())));
        }
        return result;
    }

    /**
     * 获取快速计算值
     *
     * @param entity
     * @return
     */
    public ServerResponse<AltaAnalysisCalcVo> fastCalcResult(HeaderUserInfo userInfo, AltaFastCalculateDto entity) {
        String analysisId;
        analysisId = entity.getAnalysisId();
//        AltaAnalysisCalcResult analysisCalcResult = new AltaAnalysisCalcResult();
        // 查询出当前分析的设置数据
        AltaSetting setting = settingMapper.selectById(analysisId);
        // 获取当前计算的常规计算数据，若无分析数据则无法进行快速计算
        AltaCalcResult calcResult = calcResultMapper.selectOne(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId).eq(AltaCalcResult::getDistributionType, setting.getDistributionType()).eq(AltaCalcResult::getLifeStressModel, setting.getLifeStressModel()));
        if (calcResult == null) {
            return ServerResponse.createBySuccess();
        }
        // 组装快速计算的请求参数
        String uri = rgaApiService.getQuickCalcUri(entity.getQuickCalcType(), entity);
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("results", AltaCalcResultApiResp.getAltaCalcResultApiResp(calcResult));
        // 获取当前选择的应力值
        List<Double> useStress = columnMapper.selectUseStress(analysisId);
        requestMap.put("useStress", useStress);
        requestMap.put("highStress", new Integer[0]);
        if (entity.getQuickCalcType().equals(9)) {
            // 若选择的计算模型为加速因子则需要添加HighStress请求参数
            List<Double> highStress = entity.getSelectColumn().stream().map(AltaColumn::getStressLevel).collect(Collectors.toCollection(() -> new ArrayList<>(entity.getSelectColumn().size())));
            requestMap.put("highStress", highStress);
        }

        List<AltaCalcResultVo> result = new ArrayList<>();
        // 调用郭博士的快速计算接口进行快速计算
        String resultsCallApi = rgaApiService.getResultsCallApi(new JSONObject(), uri, AltaConstants.RGA_API_CONTENTTYPE_1, AltaConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(requestMap));
        if (resultsCallApi.contains("error") || StringUtils.isBlank(resultsCallApi)) {
            result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.计算结果, userInfo.getLanguageKey()), AltaCustomPromptEnum.getLanguageMessageByKey(AltaCustomPromptEnum.无法计算, userInfo.getLanguageKey())));
        } else {
            result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.计算结果, userInfo.getLanguageKey()), getScientificResult(resultsCallApi)));
        }
        // 若计算模型不是特征寿命调用郭博士上下限的计算接口进行计算
        if (!entity.getQuickCalcType().equals(0)) {
            uri = rgaApiService.getQuickCalcBoundsUri(entity.getQuickCalcType(), entity);
            String resultsBoundsCallApi = rgaApiService.getResultsCallApi(new JSONObject(), uri, AltaConstants.RGA_API_CONTENTTYPE_1, AltaConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(requestMap));
            if (resultsBoundsCallApi.contains("error") || StringUtils.isBlank(resultsBoundsCallApi)) {
                if (!"bottom".equals(entity.getSided())) {
                    result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.上限, userInfo.getLanguageKey()), AltaCustomPromptEnum.getLanguageMessageByKey(AltaCustomPromptEnum.无法计算, userInfo.getLanguageKey())));
                }
                if (!"top".equals(entity.getSided())) {
                    result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.下限, userInfo.getLanguageKey()), AltaCustomPromptEnum.getLanguageMessageByKey(AltaCustomPromptEnum.无法计算, userInfo.getLanguageKey())));
                }
            } else {
                String[] upAndDown = resultsBoundsCallApi.replace("[", "").replace("]", "").split(",");
                if (!"bottom".equals(entity.getSided())) {
                    result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.上限, userInfo.getLanguageKey()), getScientificResult(upAndDown[1])));
                }
                if (!"top".equals(entity.getSided())) {
                    result.add(new AltaCalcResultVo(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.下限, userInfo.getLanguageKey()), getScientificResult(upAndDown[0])));
                }
            }
        }
//        StringBuilder confidenceParam = new StringBuilder();
//        StringBuilder calcMethodParam = new StringBuilder();
//        // 生成置信区间类型("None","TwoSided","Top","BothOneSided","Bottom")
//        if (!AltaConfidenceTypeCodeEnum.无.getIndex().equals(entity.getSided())) {
//            confidenceParam.append(Double.parseDouble(entity.getCl()) * 100 + "% / " + AltaConfidenceTypeCodeEnum.getValue(entity.getSided()));
//        }
//        analysisCalcResult.setConfidenceParam(confidenceParam.toString());
//        analysisCalcResult.setCalcMethodParam(entity.getSettings());
//        analysisCalcResult.setAnalysisId(entity.getAnalysisId());
//        analysisCalcResult.setPreCalcResult(entity.getCalcResult());
//        StringBuilder calcResults = new StringBuilder();
//        for (AltaCalcResultVo resultVo : result) {
//            calcResults.append(resultVo.getColumnName() + ":" + resultVo.getColumnValue() + " ");
//        }
//        analysisCalcResult.setPreSettings(entity.getPreSettings());
//        analysisCalcResult.setCalcResult(calcResults.toString());
//        analysisCalcResult.setInsertTime(new Date());
//        analysisCalcResultMapper.insert(analysisCalcResult);
        AltaAnalysisInfoService.updateUpdateInfo(userInfo, analysisMapper.selectById(analysisId).getAltaId(),
                analysisInfoMapper);
        return ServerResponse.createBySuccess(AltaAnalysisCalcVo.builder().calcResults(result).analysisCalcResults(calcResult(analysisId)).build());
    }

    public List<AltaAnalysisCalcResultVo> calcResult(String analysisId) {
        List<AltaAnalysisCalcResult> analysisCalcResults = analysisCalcResultMapper.selectList(new QueryWrapper<AltaAnalysisCalcResult>().lambda().eq(AltaAnalysisCalcResult::getAnalysisId, analysisId).orderByDesc(AltaAnalysisCalcResult::getInsertTime).last("limit 10"));
        return analysisCalcResults.stream().map(analysisCalcResult -> AltaAnalysisCalcResultVo.builder().calcSetting(analysisCalcResult.getConfidenceParam()).preSetting(analysisCalcResult.getPreSettings()).calcResult1(analysisCalcResult.getPreCalcResult()).calcResult2(analysisCalcResult.getCalcResult()).dateStr(DateTimeUtil.dateToStr(analysisCalcResult.getInsertTime())).build()).collect(Collectors.toCollection(() -> new ArrayList<>(analysisCalcResults.size())));
    }

    /**
     * 处理科学记数法的值
     *
     * @param value
     * @return
     */
    public static String getScientificResult(String value) {
        if (StringUtils.isBlank(value)) {
            return value;
        }
        if (value.contains("Infinity")) {
            return value.replaceAll("\"", "");
        }
        String eValue = "";
        String doubleValue;
        if (value.contains("E")) {
            eValue = "E" + value.split("E")[1];
            doubleValue = value.split("E")[0];
        } else if (value.contains("0.0000") && (value.contains("1") || value.contains("2") || value.contains("3")
                || value.contains("4") || value.contains("5") || value.contains("6") || value.contains("7")
                || value.contains("8") || value.contains("9"))) {
            String num = value.split("[.]")[1];
            int a = 0;
            while (num.startsWith("0")) {
                a++;
                num = num.substring(1, num.length() - 1);
            }
            num = num.length() > 5 ? num.substring(0, 5) : num;
            num = num.charAt(0) + "." + num.substring(1);
            num = value.startsWith("-") ? "-" + num : num;
            return num + "E" + "-" + (a + 1);
        } else {
            doubleValue = value;
        }
        return new BigDecimal(doubleValue).setScale(4, RoundingMode.HALF_UP) + eValue;
    }
}
