package com.nationrel.modules.alta.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.alta.mapper.*;
import com.nationrel.modules.alta.constant.AltaConstants;
import com.nationrel.modules.alta.dto.AltaAnalysisDataSaveDto;
import com.nationrel.modules.alta.dto.AltaColumnsEditDto;
import com.nationrel.modules.alta.entity.*;
import com.nationrel.modules.alta.enums.AltaDistributingCodeEnum;
import com.nationrel.modules.alta.enums.AltaLifeStressModelEnum;
import com.nationrel.modules.alta.util.JDBC2DataUtil;
import com.nationrel.modules.alta.vo.AltaAnalysisSetDataVo;
import com.nationrel.modules.alta.vo.AltaCalcResultVo;
import com.nationrel.modules.alta.vo.AltaColumnVo;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.alta.AltaCustomPromptEnum;
import com.nationrel.modules.alta.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.alta.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.alta.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.alta.general.exception.DescribeException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.nationrel.modules.alta.util.NumberParseUtil;

import java.util.*;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaColumnAndDataService {

    @Value("${spring.datasource.dynamic.datasource.master.url}")
    private String mysqlUrl;
    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String mysqlUserName;
    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String mysqlPassword;
    @Value("${alta.data-number}")
    private int dataNumber;
    private final AltaDataMapper dataMapper;
    private final AltaPlotMapper plotMapper;
    private final AltaColumnMapper columnMapper;
    private final AltaSettingMapper settingMapper;
    private final AltaCalcResultMapper calcResultMapper;
    private final AltaValidatorService validatorService;
    private final AltaAnalysisMapper analysisMapper;
    private final AltaAnalysisInfoMapper analysisInfoMapper;

    /**
     * 分页查询数据分析数据
     *
     * @param analysisId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public ServerResponse<AltaAnalysisSetDataVo> listDataByPage(String analysisId, int pageIndex, int pageSize) {
        AltaSetting setting = settingMapper.selectById(analysisId);
        List<AltaColumn> columns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda()
                .eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getSelectStatus, 1));
        List<AltaColumnVo> columnVos = new ArrayList<>(columns.size());
        columns.forEach(column -> {
            column.setName(column.getName() + "(" + column.getUnit() + ")");
            AltaColumnVo columnVo = new AltaColumnVo();
            BeanUtils.copyProperties(column, columnVo);
            columnVos.add(columnVo);
        });
        // 创建基础查询条件
        List<AltaData> list = dataMapper.selectList(new QueryWrapper<AltaData>().lambda().eq(AltaData::getAnalysisId, analysisId));
        // 简化数据以减少网络数据传输
        List<String[]> datas = new ArrayList<>(list.size());
        list.forEach(data -> datas.add(AltaData.toStringStr(data)));
        return ServerResponse.createBySuccess(new AltaAnalysisSetDataVo(setting, datas, columnVos, analysisMapper.selectById(analysisId).getEndTimeName(), dataNumber));
    }

    /**
     * 获取计算结果数据
     *
     * @param analysisId
     * @return
     */
    public ServerResponse<List<AltaCalcResultVo>> getCalcResult(String analysisId) {
        AltaSetting setting = settingMapper.selectById(analysisId);
        AltaCalcResult calcResult = calcResultMapper.selectOne(
                new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, setting.getAnalysisId())
                        .eq(AltaCalcResult::getLifeStressModel, setting.getLifeStressModel())
                        .eq(AltaCalcResult::getDistributionType, setting.getDistributionType()));
        return ServerResponse.createBySuccess(AltaCalcResultService.getcalcResultVos(setting.getLifeStressModel(),
                setting.getDistributionType(), calcResult));
    }

    /**
     * 查询加速数据分析应力设置
     *
     * @param analysisId
     * @return
     */
    public ServerResponse<List<AltaColumn>> listColumn(String analysisId, Integer languageKey) {
        QueryWrapper<AltaColumn> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(AltaColumn::getAnalysisId, analysisId);
        return ServerResponse.createBySuccess(columnMapper.selectList(queryWrapper));

    }

    /**
     * 修改应力项
     *
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> updateColumns(HeaderUserInfo userInfo, AltaColumnsEditDto entity) {
        if (CollectionUtils.isEmpty(entity.getSelectColumns())){
            return ServerResponse.createByErrorCodeMessage(ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex(), AltaCustomPromptEnum.getLanguageMessageByKey(AltaCustomPromptEnum.至少选择一个应力, userInfo.getLanguageKey()));
        }
        // 删除原有应力项配置
        columnMapper.delete(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId,
                entity.getAllColumns().get(0).getAnalysisId()));
        // 获取全部的应力项
        List<AltaColumn> result = entity.getAllColumns();
        if (CollectionUtils.isEmpty(entity.getSelectColumns())) {
            columnMapper.insertBatch(result);
        } else {
            // 处理选中的应力项
            for (AltaColumn column : result) {
                column.setSelectStatus(0);
                for (AltaColumn selectColumn : entity.getSelectColumns()) {
                    if (column.getColumnId().equals(selectColumn.getColumnId())) {
                        column.setSelectStatus(1);
                    }
                }
            }
            columnMapper.insertBatch(result);
        }
        // 删除所有的echart图信息
        plotMapper.delete(new QueryWrapper<AltaPlot>().lambda().eq(AltaPlot::getAnalysisId, entity.getAllColumns().get(0).getAnalysisId()));
        AltaAnalysisInfoService.updateUpdateInfo(userInfo, analysisMapper.selectById(result.get(0).getAnalysisId()).getAltaId(), analysisInfoMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 保存数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> updateData(HeaderUserInfo userInfo, AltaAnalysisDataSaveDto entity) {
        String analysisId = entity.getAnalysisId();
        // ==================
        // 1.修改寿命数据分析数据表信息
        // ==================
        AltaSetting setting = settingMapper.selectById(analysisId);
        boolean hasChangeSetting = !setting.getRightFlag().equals(entity.getRightFlag()) || !setting.getLeftFlag().equals(entity.getLeftFlag()) || !setting.getGroupingFlag().equals(entity.getGroupingFlag());
        // 包含删失数据(右删失数据)
        if ("0".equals(entity.getRightFlag()) && !"0".equals(setting.getRightFlag())) {
            // 开关关闭，则需要将所有数据的F/S置为S
            dataMapper.updatefByAnalysisId(analysisId);
        }
        // 包含删失数据(左删失数据)
        if ("0".equals(entity.getLeftFlag()) && !"0".equals(setting.getLeftFlag())) {
            // 开关关闭，则需要将所有数据的最近检查时间置为空
            dataMapper.updateLitByAnalysisId(analysisId);
        }
        // 分组数据关闭
        if ("0".equals(entity.getGroupingFlag()) && !"0".equals(setting.getGroupingFlag())) {
            // 开关关闭，则需要将所有数据的数量置为1
            dataMapper.updateQuantityByAnalysisId(analysisId);
        }
        // 删除原有的计算结果以及画图设置
        deleteAnalysisResultData(setting.getAnalysisId(), calcResultMapper, settingMapper);
        if (hasChangeSetting) {
            setting.setLeftFlag(entity.getLeftFlag());
            setting.setRightFlag(entity.getRightFlag());
            setting.setUpdateBy(userInfo.getUserName());
            setting.setGroupingFlag(entity.getGroupingFlag());
            setting.setUpdateTime(new Date());
            // 寿命数据分析数据表修改
            settingMapper.updateById(setting);
        }
        if (entity.getHotDataChange() > 1) {
            List<AltaData> analysisDataDetails = new ArrayList<>();
            // 解析新的数组结构的数据，将其解析为对象
            for (int i = 0; i < entity.getAnalysisDatas().size(); i++) {
                String[] data = entity.getAnalysisDatas().get(i);
                if (data[0] != null && data[1] != null) {
                    // 判断填入数值是否为大于0的数字
                    if (!NumberParseUtil.isNumeric(data[4]) || NumberParseUtil.parseDouble(data[4]) <= 0) {
                        throw new DescribeException((i + 1), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                    }
                    // 判断最近检查时间填入数值是否为大于0的数字
                    if (entity.getLeftFlag().equals(EwbConstants.FLAG_CHECKED) && (!NumberParseUtil.isNumeric(data[2]) || NumberParseUtil.parseDouble(data[2]) <= 0)) {
                        throw new DescribeException((i + 1), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.最近检查时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                    }
                    // 判断最近检查时间是否合法
                    if (entity.getLeftFlag().equals(EwbConstants.FLAG_CHECKED) && NumberParseUtil.parseDouble(data[2]) > NumberParseUtil.parseDouble(data[4])) {
                        throw new DescribeException((i + 1), ReliaqubeParamDataCheckEnum.结束时间必须大于等于最近检查时间, userInfo.getLanguageKey());
                    }
                    analysisDataDetails.add(AltaData.toAltaData(data, entity.getAnalysisId()));
                }
            }
            validatorService.validatorAnalysisDetailData(analysisDataDetails, entity.getGroupingFlag(), entity.getLeftFlag(), entity.getRightFlag(), entity.getStartDataNo(), userInfo.getLanguageKey());
            saveData(analysisId, analysisDataDetails, entity.getLeftFlag(), dataMapper, plotMapper, calcResultMapper, userInfo.getLanguageKey());
            analysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserName(), entity.getAltaId());
        }
        // 返回结果
        return ServerResponse.createBySuccess();
    }

    /**
     * 删除中间数据
     *
     * @param analysisId
     */
    public static void deleteAnalysisResultData(String analysisId, AltaCalcResultMapper calcResultMapper, AltaSettingMapper settingMapper) {
        // 删除计算结果数据
        calcResultMapper.deleteByAnalysisId(analysisId);
//        settingMapper.deleteAllCalcResult(settingId);
        // 删除xy区间信息
        settingMapper.deleteMarkSetting(analysisId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveData(String analysisId, List<AltaData> dataList, String leftFlag, AltaDataMapper dataMapper, AltaPlotMapper plotMapper, AltaCalcResultMapper calcResultMapper, Integer languageKey) {
        if (dataList != null && dataList.size() > 0) {
            // 删除集合中WeiMaterial对象属性fratios为空的对象s
            dataList.removeIf(weiMaterial -> weiMaterial.getFailureEndTime() == null);
        }

        // 插入起始数据编号
        int newStartDataNo = AltaConstants.NUM_ONE;
        if (dataList != null && dataList.size() > 0) {
            // ==================
            // 3.保存寿命数据分析明细数据
            // ==================
            // 分析明细数据处理
            List<AltaData> dataDetails = new ArrayList<>();
            for (AltaData dataDetail : dataList) {
                if (dataDetail != null) {
                    if (AltaConstants.FLAG_CHECKED.equals(leftFlag) && dataDetail.getLastInspectTime() == null) {
                        throw new DescribeException(newStartDataNo, ReliaqubeParamDataCheckEnum.最近检查时间不可为空, languageKey);
                    }
                    String fs = dataDetail.getFailureSuspension();
                    // ==================
                    // 判断明细数据的数据类型
                    // ==================
                    if (AltaConstants.STATE_SUSPENSION.equalsIgnoreCase(fs)) {
                        // 若状态为s，则数据类型为右删失数据类型
                        dataDetail.setFailureSuspension(AltaConstants.STATE_S);
                        // dataDetail.setFailureEndTime(dataDetail.getFailureEndTime());
                    } else {
                        dataDetail.setFailureSuspension(AltaConstants.STATE_F);
                        // 对于左删失，区间，完整数据，需要计算平均时间
                        // dataDetail.setFailureEndTime((dataDetail.getFailureEndTime() +
                        // lastInspectTime) / 2);
                    }
                    if (null == dataDetail.getQuantity()) {
                        dataDetail.setQuantity(1);
                    } else if (dataDetail.getQuantity().equals(0)) {
                        throw new DescribeException(newStartDataNo, ReliaqubeParamDataCheckEnum.数量不可为空, languageKey);
                    }
                    // 数据编号
                    dataDetail.setDataNo(newStartDataNo);
                    newStartDataNo++;
                    // 数据ID
                    dataDetail.setAnalysisId(analysisId);
                    dataDetails.add(dataDetail);
                }
            }
            // ==================
            // 1.删除当前页面分析明细数据表数据
            // ==================
            // 删除所有的echart图信息
            plotMapper.delete(new QueryWrapper<AltaPlot>().lambda().eq(AltaPlot::getAnalysisId, analysisId));
            // 删除所有的计算信息
            calcResultMapper.delete(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId));
            JDBC2DataUtil.batchInsert(dataDetails, mysqlUrl, mysqlUserName, mysqlPassword);
        }
    }

    public ServerResponse<Map> listSelectColumn(String analysisId, Integer languageKey) {
        Map map = new HashMap();
        List<AltaColumn> columns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getShowStatus, 1));
        map.put("column", columns);
        AltaSetting setting = settingMapper.selectById(analysisId);
        String analysisSetting = AltaLifeStressModelEnum.getNameByLanguageKey(setting.getLifeStressModel().toString(), languageKey) + "; " + AltaDistributingCodeEnum.getNameByLanguageKey(setting.getDistributionType().toString(), languageKey) + "; ";
        for (AltaColumn column : columns) {
            analysisSetting = analysisSetting + column.getName() + "(" + column.getUnit() + ")" + ":" + column.getStressLevel() + "; ";
        }
        map.put("analysisSetting", analysisSetting);
        return ServerResponse.createBySuccess(map);
    }
}
