// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.alta.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.alta.mapper.*;
import com.nationrel.modules.alta.constant.AltaConstants;
import com.nationrel.modules.alta.constant.AltaDistributionTypeEnum;
import com.nationrel.modules.alta.constant.AltaPlotTypeEnum;
import com.nationrel.modules.alta.dto.AltaGroupNameValueDto;
import com.nationrel.modules.alta.dto.AltaImageDetailDto;
import com.nationrel.modules.alta.entity.*;
import com.nationrel.modules.alta.enums.AltaConfidenceTypeCodeEnum;
import com.nationrel.modules.alta.enums.AltaParamDataExceptionEnum;
import com.nationrel.modules.alta.util.NumberParseUtil;
import com.nationrel.modules.alta.vo.AltaCalcResultsVo;
import com.nationrel.modules.alta.vo.AltaPlotVo;
import com.nationrel.modules.alta.vo.AltaSelectColumnVo;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.alta.AltaCommonUsedEnum;
import com.nationrel.modules.alta.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.alta.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.alta.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.alta.general.exception.DescribeException;
import com.nationrel.modules.alta.general.util.common.DateTimeUtil;
import com.nationrel.modules.alta.general.util.common.ValidateUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName AltaPlotService
 * @Description 生成图像信息
 * @Author hcs
 * @CreateDate 2021-05-27
 * @Version V1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaPlotService {

    @Value("${relia-qube-url}")
    private String reliaQubeUrl;
    private final AltaAnalysisInfoMapper analysisInfoMapper;
    private final AltaCalcResultMapper calcResultMapper;
    private final AltaAnalysisMapper analysisMapper;
    private final AltaSettingMapper settingMapper;
    private final AltaRgaApiService rgaApiService;
    private final AltaColumnMapper columnMapper;
    private final AltaDataMapper dataMapper;

    /**
     * Alta画图
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public ServerResponse<AltaPlotVo> selectEchartDetail(HeaderUserInfo userInfo, AltaImageDetailDto entity) {
        String analysisId = entity.getAnalysisId();
        // 取出现有的分析、设置、计算结果信息
        AltaSetting setting = settingMapper.selectById(analysisId);
        AltaAnalysis analysis = analysisMapper.selectById(analysisId);
        AltaCalcResult altaCalcResult = calcResultMapper.selectOne(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId)
                .eq(AltaCalcResult::getDistributionType, setting.getDistributionType())
                .eq(AltaCalcResult::getLifeStressModel, setting.getLifeStressModel()));
        // 没有计算结果则需要计算AltaParamDataExceptionEnum
        if (altaCalcResult == null) {
            return ServerResponse.createByErrorMessage(AltaParamDataExceptionEnum.getLanguageMessageByKey(AltaParamDataExceptionEnum.没有计算结果无法画图, userInfo.getLanguageKey()));
        }

        // 获取当前选择的列
        List<AltaColumn> columns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getShowStatus, "1"));
        List<String> streeName = new ArrayList<>();
        String columnIds = "";
        String columnIdQuerys = "";
        for (AltaColumn column : columns) {
            streeName.add(column.getName());
            columnIds = StringUtils.isBlank(columnIds) ? column.getColumnId() : columnIds + "," + column.getColumnId();
            columnIdQuerys = StringUtils.isBlank(columnIdQuerys) ? column.getColumnId() : columnIdQuerys + ",'/'," + column.getColumnId();
        }
        columnIdQuerys = columnIdQuerys + ", '/'";
        // 数据库存在则直接返回
        // 如果设置有变化则重新计算
        AltaPlotVo result = new AltaPlotVo();
        BeanUtils.copyProperties(entity, result);
        result.setDistributionType(setting.getDistributionType());
        result.setEndTimeName(analysis.getEndTimeName());
        String confidenceLevel = "None".equals(entity.getBoundSided()) ? AltaConfidenceTypeCodeEnum.getNameByLanguageKey(AltaConfidenceTypeCodeEnum.无.getIndex(), userInfo.getLanguageKey()) + "；" : Double.parseDouble(entity.getConfidenceLevel()) * 100 + "%；";
        result.setConfidenceDetail(AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.置信类型, userInfo.getLanguageKey()) + "：" + AltaConfidenceTypeCodeEnum.getNameByLanguageKey(entity.getBoundSided(), userInfo.getLanguageKey()) + "；" + AltaCommonUsedEnum.getLanguageMessageByKey(AltaCommonUsedEnum.置信度, userInfo.getLanguageKey()) + "：" + confidenceLevel);
        result.setDates(new String[]{DateTimeUtil.dateToStr(new Date(), "yyyy-MM-dd"), DateTimeUtil.dateToStr(new Date(), "HH:mm:ss")});
        if (entity.getPlotType() == 8) {
            List<AltaSelectColumnVo> selectColumnVos = columns.stream().map(column -> AltaSelectColumnVo.builder().columnId(column.getColumnId()).name(column.getName() + "(" + column.getUnit() + ")").minValue(dataMapper.selectMinValue(column.getAnalysisId(), column.getColumnId())).maxValue(dataMapper.selectMaxValue(column.getAnalysisId(), column.getColumnId())).build()).collect(Collectors.toList());
            // 获取最大最小的应力值
            result.setPlotType(entity.getPlotType());
            result.setSelectColumnVos(selectColumnVos);
            result.setCalcResult(altaCalcResult);
            result.setLifeStressModel(setting.getLifeStressModel());
        } else {
            AltaAnalysisInfoService.updateUpdateInfo(userInfo, analysis.getAltaId(), analysisInfoMapper);
            // 准备数据
            List<AltaData> datas = dataMapper.selectList(new QueryWrapper<AltaData>().lambda().eq(AltaData::getAnalysisId, analysisId));
            // 数据库有计算结果
            Map<String, Object> dataRow = rgaApiService.assemblyData(columns, datas, setting);
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("data", dataRow);

            requestMap.put("results", AltaCalcResultApiResp.getAltaCalcResultApiResp(altaCalcResult));
            List<Double> useStress = columns.stream().map(AltaColumn::getStressLevel).collect(Collectors.toList());
            requestMap.put("useStress", useStress);

            JSONObject jsonObject = new JSONObject();
            String failureRank = rgaApiService.getResultsCallApi(jsonObject, reliaQubeUrl + AltaConstants.FAILURE_RANK,
                    AltaConstants.RGA_API_CONTENTTYPE_1, AltaConstants.RGA_API_ACCEPTTYPE_1,
                    JSONObject.toJSONString(requestMap));
            dataRow.put("failureRank", JSONArray.parse(failureRank));
            requestMap.put("stressIndicator", 0);
            requestMap.put("stressName", streeName);
            JSONObject settings = new JSONObject();
            settings.put("resolution", "High");
            requestMap.put("settings", settings);
            JSONObject bounds = new JSONObject();
            bounds.put("confidenceLevel", entity.getConfidenceLevel().equals(AltaDistributionTypeEnum.LogNormal.toString()) ? 7 : Double.parseDouble(entity.getConfidenceLevel()));
            bounds.put("method", "FisherMatrix");
            bounds.put("sided", entity.getBoundSided());
            bounds.put("on", "Function");
            settings.put("bounds", bounds);
            JSONObject xAxis = new JSONObject();
            // 非水平概率图和分布概率图时，如果未传入xy区间，说明第一次进入，则获取全部计算数据
            if (AltaPlotTypeEnum.TestLevelProbability.getIndex().equals(entity.getPlotType()) || AltaPlotTypeEnum.UseLevelProbability.getIndex().equals(entity.getPlotType())) {
                // 如果前端未传入xy区间，则说明是初始化第一次进入页面，需要计算xy最大和最小的x值
                if (entity.getXaxisStart() != null || setting.getXaxisStart() != null) {
                    xAxis.put("AutoScale", false);
                    xAxis.put("Minimum", entity.getXaxisStart() == null ? setting.getXaxisStart() : entity.getXaxisStart());
                    xAxis.put("Maximum", entity.getXaxisEnd() == null ? setting.getXaxisEnd() : entity.getXaxisEnd());
                    result.setXaxisStart(entity.getXaxisStart() == null ? setting.getXaxisStart() : entity.getXaxisStart());
                    result.setXaxisEnd(entity.getXaxisEnd() == null ? setting.getXaxisEnd() : entity.getXaxisEnd());
                    result.setYaxisStart(entity.getYaxisStart() == null ? setting.getYaxisStart() : entity.getYaxisStart());
                    result.setYaxisEnd(entity.getYaxisEnd() == null ? setting.getYaxisEnd() : entity.getYaxisEnd());
                    settings.put("XAxis", xAxis);
                }
            }
            String uri = rgaApiService.getPlotUri(setting.getDistributionType(), entity.getPlotType());
            // 4.3.3 Life vs Stress Plot Example
            lifeStressParam(requestMap, entity.getPlotType(), !entity.getBoundSided().equals(AltaConfidenceTypeCodeEnum.无.getIndex()));
            String plotResult = rgaApiService.getResultsCallApi(jsonObject, uri, AltaConstants.RGA_API_CONTENTTYPE_1, AltaConstants.RGA_API_ACCEPTTYPE_1, JSONObject.toJSONString(requestMap));
            // 返回值有错误，画图异常
            if (plotResult.indexOf("error") < 0) {
                result.setAnalysisData(JSONObject.parseObject(plotResult));
            } else {
                throw new DescribeException(ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, userInfo.getLanguageKey(), ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex());
            }
        }
        // F/S数据
        if (entity.getPlotType() == 7) {
            List<String> columnGroups = dataMapper.selectGroups(columnIds, columnIdQuerys, analysisId);
            List<AltaGroupNameValueDto> columnGroupFqs = dataMapper.selectGroupSumByFs(columnIds, columnIdQuerys, analysisId, "1");
            List<AltaGroupNameValueDto> columnGroupSqs = dataMapper.selectGroupSumByFs(columnIds, columnIdQuerys, analysisId, "0");
            List<String> columnGroupes = new ArrayList<>(columnGroupFqs.size());
            List<String> columnGroupSums = new ArrayList<>(columnGroupFqs.size());
            for (int i = 0; i < columnGroups.size(); i++) {
                AltaGroupNameValueDto columnGroupFq = null;
                AltaGroupNameValueDto columnGroupSq = null;
                String columnGroup = columnGroups.get(i);
                for (AltaGroupNameValueDto columnGroupF : columnGroupFqs) {
                    if (columnGroup.equals(columnGroupF.getName())) {
                        columnGroupFq = columnGroupF;
                    }
                }
                for (AltaGroupNameValueDto columnGroupS : columnGroupSqs) {
                    if (columnGroup.equals(columnGroupS.getName())) {
                        columnGroupSq = columnGroupS;
                    }
                }
                if (columnGroup.contains(".000000")) {
                    columnGroup = columnGroup.replaceAll(".000000", "");
                } else {
                    columnGroup = !columnGroup.contains(".") ? columnGroup : columnGroup.replaceAll("00000/", "/");
                    columnGroup = !columnGroup.contains(".") ? columnGroup : columnGroup.replaceAll("0000/", "/");
                    columnGroup = !columnGroup.contains(".") ? columnGroup : columnGroup.replaceAll("000/", "/");
                    columnGroup = !columnGroup.contains(".") ? columnGroup : columnGroup.replaceAll("00/", "/");
                    columnGroup = !columnGroup.contains(".") ? columnGroup : columnGroup.replaceAll("0/", "/");
                }
                columnGroup = columnGroup.substring(0, columnGroup.length() - 1);
                columnGroupes.add(columnGroup);
                Integer fq = columnGroupFq == null ? 0 : columnGroupFq.getValue();
                Integer sq = columnGroupSq == null ? 0 : columnGroupSq.getValue();
                columnGroupSums.add("F: " + fq + "/S: " + sq);
            }
            result.setAnalysisColumn(columnGroupes.toArray(new String[columnGroupes.size()]));
            result.setAnalysisColumnFS(columnGroupSums.toArray(new String[columnGroupSums.size()]));
        } else {
            result.setFailureQ(dataMapper.selectSumByForS(analysisId, "1"));
            result.setSuspensionQ(dataMapper.selectSumByForS(analysisId, "0"));
            result.setFailureQ(result.getFailureQ() == null ? 0 : result.getFailureQ());
            result.setSuspensionQ(result.getSuspensionQ() == null ? 0 : result.getSuspensionQ());
        }
        //辅助线画图数据
        if (entity.getPlotType() == 7 || entity.getPlotType() == 2) {
            result.setTime(setting.getTime());
            result.setUnReliability(setting.getUnReliability());
            result.setMuMark(setting.getMuMark());
            result.setSigmaMark(setting.getSigmaMark());
            result.setTargetReliability(setting.getTargetReliability());
            result.setTargetTime(setting.getTargetTime());
            result.setBetaMark(setting.getBetaMark());
            result.setEtaMark(setting.getEtaMark());
        }
        AltaCalcResultsVo altaCalcResultsVo = new AltaCalcResultsVo();
        BeanUtils.copyProperties(altaCalcResult, altaCalcResultsVo);
        result.setCalcResults(altaCalcResultsVo);
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 最大最小x的中间计算
     *
     * @param analysisId
     * @return
     */
    private List<Double> calcMinMaxX(String analysisId) {
        List<Double> result = new ArrayList<>();
        // 计算最小的x
        result.add(calcMinX(analysisId));
        // 计算最大的x
        result.add(calcMaxX(analysisId));
        if (result.get(1) / result.get(0) == 10) {
            // 如果xmin和xmax仅相差一个区间，则扩大区间保证画出的图形饱满一些
            result.set(0, result.get(0) / 10);
            result.set(1, result.get(0) * 10);
        }
        return result;
    }

    /**
     * 计算当前数据的最小x
     *
     * @param analysisId
     * @return
     */
    private Double calcMinX(String analysisId) {
        Double minx = dataMapper.selectMinXvalue(analysisId);
        if (minx < 0.0001) {
            minx = 0.0001;
        } else {
            for (double i = 0.0001; i < minx; i = i * 10) {
                if (i < minx && i * 10 >= minx) {
                    return i;
                }
            }
        }
        return minx;
    }


    /**
     * 计算当前数据的最大x
     *
     * @param analysisId
     * @return
     */
    private Double calcMaxX(String analysisId) {
        Double max = dataMapper.selectMaxXvalue(analysisId);
        if (max > 1000000000) {
            max = 1000000000.0;
        } else {
            for (double i = 1000000000.0; i > max; i = i / 10) {
                if (i > max && i / 10 <= max) {
                    return i;
                }
            }
        }
        return max;
    }

    /**
     * 拼接查询条件
     *
     * @param resultMap
     * @param plotType
     */
    private void lifeStressParam(Map<String, Object> resultMap, Integer plotType, boolean hasBound) {
        if (plotType.equals(AltaPlotTypeEnum.LifeVsStress.getIndex())) {
            Map<String, Object> lifeSettings = new HashMap<>();
            Map<String, Object> charLifeSetting = new HashMap<>();
            charLifeSetting.put("showline", true);
            charLifeSetting.put("showbounds", false);
            Map<String, Object> percentileSetting = new HashMap<>();
            percentileSetting.put("percentile", new double[]{0.5});
            percentileSetting.put("showline", false);
            percentileSetting.put("showbounds", false);
            Map<String, Object> failurePointsSetting = new HashMap<>();
            failurePointsSetting.put("showPoints", true);
            failurePointsSetting.put("showPdf", true);
            Map<String, Object> meanLifeSetting = new HashMap<>();
            // 控制开关线
            meanLifeSetting.put("showline", false);
            // 控制双侧开关线
            meanLifeSetting.put("showbounds", true);

            lifeSettings.put("CharLifeSetting", charLifeSetting);
            lifeSettings.put("PercentileSetting", percentileSetting);
            lifeSettings.put("FailurePointsSetting", failurePointsSetting);
            lifeSettings.put("MeanLifeSetting", meanLifeSetting);
            lifeSettings.put("pdfCurveScalingFactor", 0.2);
            resultMap.put("LifeSettings", lifeSettings);
        } else if (plotType.equals(AltaPlotTypeEnum.TestLevelProbability.getIndex())) {
            Map<String, Object> testStressSetting = new HashMap<>();
            testStressSetting.put("showline", true);
            testStressSetting.put("showbounds", hasBound);
            Map<String, Object> useStressSetting = new HashMap<>();
            useStressSetting.put("showline", true);
            useStressSetting.put("showbounds", hasBound);
            resultMap.put("TestStressSetting", testStressSetting);
            resultMap.put("UseStressSetting", useStressSetting);
        }
    }

    /**
     * 更新画图的标记信息
     *
     * @param param
     * @return
     */
    public ServerResponse updateSetting(AltaSetting param, Integer languageKey) {
        AltaSetting altaSetting = settingMapper.selectById(param.getAnalysisId());
        // 将前端传入的最新的设置更新至原有设置中
        if (param.getXaxisStart() != null) {
            // 更新画图端的设置信息
            // 判断前端传入的不可靠度是否合法
            if (param.getUnReliability() != null) {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.不可靠度, languageKey), String.valueOf(param.getUnReliability()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
            }
            // 判断前端传入的不可靠度是否合法
            if (param.getYaxisEnd() != null) {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.Y, languageKey), String.valueOf(param.getYaxisEnd()), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
            }
            // 将前端传入的最新的设置更新至原有设置中
            altaSetting.setXaxisStart(param.getXaxisStart());
            altaSetting.setXaxisEnd(param.getXaxisEnd());
            altaSetting.setYaxisStart(param.getYaxisStart());
            altaSetting.setYaxisEnd(param.getYaxisEnd());
            altaSetting.setTime(param.getTime());
            altaSetting.setUnReliability(param.getUnReliability());
            altaSetting.setBetaMark(param.getBetaMark());
            altaSetting.setEtaMark(param.getEtaMark());
            altaSetting.setSigmaMark(param.getSigmaMark());
            altaSetting.setMuMark(param.getMuMark());
            altaSetting.setLambdaMark(param.getLambdaMark());
            altaSetting.setTargetTime(param.getTargetTime());
            altaSetting.setTargetReliability(param.getTargetReliability());
        } else if (param.getDistributionType() != null) {
            // 更新计算端的设置信息
            altaSetting.setDistributionType(param.getDistributionType());
            altaSetting.setLifeStressModel(param.getLifeStressModel());
        }
        settingMapper.updateById(altaSetting);
        return ServerResponse.createBySuccess();
    }

    /**
     * @param fieldName  待校验字段名
     * @param fieldValue 待校验字段值
     * @param min        校验范围最小值
     * @param max        校验范围最大值
     * @Title: calcParamCheck
     * @Description: 参数校验
     */
    public static void calcParamCheck(String fieldName, String fieldValue, BigDecimal min, BigDecimal max, Integer languageKey) {

        if (ValidateUtils.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtils.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        } else if (NumberParseUtil.parseBigDecimal(fieldValue).compareTo(min) <= 0 || NumberParseUtil.parseBigDecimal(fieldValue).compareTo(max) >= 0) {
            throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
        }
    }
}
